// Copyright (C) 2022 RozK
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "math.hpp"

void rk_vec3_rotate(
    rk_vec3 & ret,
    rk_vec3 const & vec3,
    rk_vec3 const & axis,
    rk_float const angle) {
    ret = glm::angleAxis(angle, axis) * vec3;
}

void rk_vec3_mul_vec3(
    rk_vec3 & ret,
    rk_vec3 const & a,
    rk_vec3 const & b) {
    ret = a * b;
}

void rk_mat3_rotation(
    rk_mat3 & ret,
    rk_vec3 const & axis,
    rk_float const angle) {
    ret = glm::mat3_cast(glm::angleAxis(angle, axis));
}

void rk_mat3_mul_vec3(
    rk_vec3 & ret,
    rk_mat3 const & a,
    rk_vec3 const & b) {
    ret = a * b;
}

void rk_mat3_mul_mat3(
    rk_mat3 & ret,
    rk_mat3 const & a,
    rk_mat3 const & b) {
    ret = a * b;
}

void rk_mat4_projection(
    rk_mat4 & ret,
    rk_float hfov,
    rk_float ratio,
    rk_float near,
    rk_float far) {
    ret = glm::perspectiveRH(hfov, ratio, near, far);
}

void rk_mat4_lookat(
    rk_mat4 & ret,
    rk_vec3 const & position,
    rk_vec3 const & lookat) {
    ret = glm::lookAtRH(position, lookat, vec3_up);
}

void rk_mat4_orbit(
    rk_mat4 & ret,
    rk_vec3 const & origin,
    rk_float const yaw,
    rk_float const pitch,
    rk_float const distance) {
    ret = glm::translate(
        glm::lookAtRH(glm::vec3(0.f, -distance, 0.f), vec3_origin, vec3_up) *
            glm::mat4(glm::angleAxis(pitch, vec3_right) * glm::angleAxis(yaw, vec3_up)),
        origin);
}

void rk_mat4_mul_vec3(
    rk_vec3 & ret,
    rk_mat4 const & a,
    rk_vec3 const & b,
    rk_float const w) {
    ret = glm::vec3(a * glm::vec4(b, w));
}

void rk_mat4_mul_vec4(
    rk_vec4 & ret,
    rk_mat4 const & a,
    rk_vec4 const & b) {
    ret = a * b;
}

void rk_mat4_mul_mat4(
    rk_mat4 & ret,
    rk_mat4 const & a,
    rk_mat4 const & b) {
    ret = a * b;
}
