// Copyright (C) 2022 RozK
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef _RK_ENGINE_MATH_H
#define _RK_ENGINE_MATH_H

#include "types.hpp"
#include <glm/glm.hpp>
#include <glm/ext.hpp>

typedef glm::vec2 rk_vec2;
typedef glm::vec3 rk_vec3;
typedef glm::vec4 rk_vec4;
typedef glm::mat3 rk_mat3;
typedef glm::mat4 rk_mat4;

#define RK_CHECK_MATH_TYPE(_t, _e, _c) static_assert(sizeof(_t) == sizeof(_e) * (_c))

RK_CHECK_MATH_TYPE(rk_vec2, float, 2);
RK_CHECK_MATH_TYPE(rk_vec3, float, 3);
RK_CHECK_MATH_TYPE(rk_vec4, float, 4);
RK_CHECK_MATH_TYPE(rk_mat3, rk_vec3, 3);
RK_CHECK_MATH_TYPE(rk_mat4, rk_vec4, 4);

#define vec3_right   (rk_vec3(1.f, 0.f, 0.f))
#define vec3_forward (rk_vec3(0.f, 1.f, 0.f))
#define vec3_up      (rk_vec3(0.f, 0.f, 1.f))
#define vec3_origin  (rk_vec3(0.f, 0.f, 0.f))

RK_EXPORT void rk_vec3_rotate(
    rk_vec3 & ret,
    rk_vec3 const & vec3,
    rk_vec3 const & axis,
    rk_float const angle);

RK_EXPORT void rk_vec3_mul_vec3(
    rk_vec3 & ret,
    rk_vec3 const & a,
    rk_vec3 const & b);

RK_EXPORT void rk_mat3_rotation(
    rk_mat3 & ret,
    rk_vec3 const & axis,
    rk_float const angle);

RK_EXPORT void rk_mat3_mul_vec3(
    rk_vec3 & ret,
    rk_mat3 const & a,
    rk_vec3 const & b);

RK_EXPORT void rk_mat3_mul_mat3(
    rk_mat3 & ret,
    rk_mat3 const & a,
    rk_mat3 const & b);

RK_EXPORT void rk_mat4_projection(
    rk_mat4 & ret,
    rk_float hfov,
    rk_float ratio,
    rk_float near,
    rk_float far);

RK_EXPORT void rk_mat4_lookat(
    rk_mat4 & ret,
    rk_vec3 const & position,
    rk_vec3 const & lookat);

RK_EXPORT void rk_mat4_orbit(
    rk_mat4 & ret,
    rk_vec3 const & origin,
    rk_float const yaw,
    rk_float const pitch,
    rk_float const distance);

RK_EXPORT void rk_mat4_mul_vec3(
    rk_vec3 & ret,
    rk_mat4 const & a,
    rk_vec3 const & b,
    rk_float const w);

RK_EXPORT void rk_mat4_mul_vec4(
    rk_vec4 & ret,
    rk_mat4 const & a,
    rk_vec4 const & b);

RK_EXPORT void rk_mat4_mul_mat4(
    rk_mat4 & ret,
    rk_mat4 const & a,
    rk_mat4 const & b);

#endif // _RK_ENGINE_MATH_H
